/* Copyright 2004, 2005 Nicholas Bishop
 * 
 * This file is part of SharpConstruct.
 *
 * SharpConstruct is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * SharpConstruct is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SharpConstruct; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef MESHDATA_HH
#define MESHDATA_HH

#include "Grid.hh"
#include "Point.hh"
#include "Polygon3D.hh"
#include "Utilities.hh"

namespace SharpConstruct
{
	class VertexData : public Resource
	{
	public:
		Point3DVector& VertexLocations()
		{
			return _vertex_locations;
		}
		Point3DVector& VertexNormals()
		{
			return _vertex_normals;
		}
	private:
		Point3DVector _vertex_locations;
		Point3DVector _vertex_normals;
	};
	class ColorData : public Resource
	{
	public:
		std::vector< Color >& VertexColors()
		{
			return _vertex_colors;
		}
	private:
		std::vector< Color > _vertex_colors;
	};
	class PolygonData : public Resource
	{
	public:
		const Point3DVector& TriangleNormals() const
		{
			return _triangle_normals;
		}
		Point3DVector& TriangleNormals()
		{
			return _triangle_normals;
		}
			
		const Point3DVector& QuadNormals() const
		{
			return _quad_normals;
		}
		Point3DVector& QuadNormals()
		{
			return _quad_normals;
		}

		const std::vector< Triangle >& Triangles() const
		{
			return _triangles;
		}
		std::vector< Triangle >& Triangles()
		{
			return _triangles;
		}

		const std::vector< Quad >& Quads() const
		{
			return _quads;
		}
		std::vector< Quad >& Quads()
		{
			return _quads;
		}

		const std::vector< PolygonUsers >& VertexUsers() const
		{
			return _vertex_users;
		}
		std::vector< PolygonUsers >& VertexUsers()
		{
			return _vertex_users;
		}

		const EdgeData& Edges() const
		{
			return edges_;
		}
		EdgeData& Edges()
		{
			return edges_;
		}

		const Grid& CurrentGrid() const
		{
			return grid_;
		}
		Grid& CurrentGrid()
		{
			return grid_;
		}
	private:
		Point3DVector _triangle_normals;
		Point3DVector _quad_normals;
		std::vector< Triangle > _triangles;
		std::vector< Quad > _quads;
		std::vector< PolygonUsers > _vertex_users;
		EdgeData edges_;
		Grid grid_;
	};
}

#endif
